(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    106508,       2813]
NotebookOptionsPosition[     64237,       1832]
NotebookOutlinePosition[    101654,       2647]
CellTagsIndexPosition[    101611,       2644]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["The Gauss Application Example", "Title"],

Cell["\<\
Copyright \[Copyright] MathCore AB 1999-2004\
\>", "Copyright"],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["\<\
The Gauss application example uses a text-book Gauss-elimination algorithm to \
solve a linear equation system. Two versions of the Gauss elimination \
algorithm are presented:\
\>", "Text"],

Cell[TextData[{
 StyleBox["GaussSolveArraySlice",
  FontWeight->"Bold"],
 ". The algorithm is largely coded using the Matlab-like array section \
operations."
}], "Itemize"],

Cell[TextData[{
 StyleBox["GaussSolveForLoops",
  FontWeight->"Bold"],
 ". The algorithm is coded using traditional for-loops."
}], "Itemize"],

Cell[TextData[{
 "Both versions of the algorithm are compiled to Fortran90 code. The \
performance is measured, both including and without MathLink overhead. \
Complete evaluation of this notebook takes 3-5 minutes on a 1.5 GHz computer. \
See ",
 StyleBox["factor",
  FontWeight->"Bold"],
 " variable below to adjust the speed."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell["Load the MathCode package", "Subsection"],

Cell["\<\
First put a Needs statement, to make sure that the MathCode application is \
loaded:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<MathCode`\>\"", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"MathCode F90 1.2.4\"\>", "\[InvisibleSpace]", "\<\" for \"\>", 
   "\[InvisibleSpace]", "\<\"g95\"\>", 
   "\[InvisibleSpace]", "\<\" loaded from \"\>", 
   "\[InvisibleSpace]", "\<\"C:\\\\b\\\\omath\\\\MathCode\"\>"}],
  SequenceForm[
  "MathCode F90 1.2.4", " for ", "g95", " loaded from ", 
   "C:\\b\\omath\\MathCode"],
  Editable->False]], "Print",
 CellChangeTimes->{3.510639778766369*^9, 3.5106413422319613`*^9, 
  3.510641549584701*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check current directory", "Subsection"],

Cell["\<\
You might want to set the directory to some place where all generated files \
are put, e.g. the one below, or some other directory.\
\>", "Text"],

Cell["\<\
SetDirectory[$MCRoot<>$PathnameSeparator<>\"DemosF90\"<>$PathnameSeparator<>\"\
Gauss\"];\
\>", "Input"],

Cell["Check whether this file is in the current directory.", "Text"],

Cell["ExistFileQ[\"GaussF90.nb\"];", "Input"],

Cell["Off[General::spell1]", "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Start the Gauss Package", "Section"],

Cell["BeginPackage[\"Gauss`\",{MathCodeContexts}];", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell["Define exported symbols", "Text"],

Cell["\<\
Begin[\"Gauss`\"];

GaussSolveArraySlice;
GaussSolveForLoops;
GaussSolveForLoopsC;

End[];\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[CellGroupData[{

Cell["Set compilation options", "Subsection"],

Cell["\<\
Possibly set compilation options. In this case no extra options are needed.\
\>", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Body of the Gauss Package", "Section"],

Cell["Begin[\"`Private`\"];", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[CellGroupData[{

Cell["The GaussSolveArraySlice function", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"GaussSolveArraySlice", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"Real", "[", 
         RowBox[{"n_", ",", "n_"}], "]"}], " ", "ain_"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Real", "[", 
         RowBox[{"n_", ",", "m_"}], "]"}], " ", "bin_"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Integer", " ", "iterations_"}]}], "]"}], "\[Rule]", 
     RowBox[{"Real", "[", 
      RowBox[{"n", ",", "m"}], "]"}]}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Real", "[", "n", "]"}], " ", "dumc"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Real", "[", 
          RowBox[{"n", ",", "n"}], "]"}], " ", "a"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Real", "[", 
          RowBox[{"n", ",", "m"}], "]"}], " ", "b"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Integer", "[", "n", "]"}], " ", 
         RowBox[{"{", 
          RowBox[{"ipiv", ",", "indxr", ",", "indxc"}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Integer", " ", 
         RowBox[{"{", 
          RowBox[{"i", ",", "k", ",", "l", ",", "irow", ",", "icol"}], 
          "}"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"Real", " ", 
         RowBox[{"{", 
          RowBox[{"pivinv", ",", "amax", ",", "tmp"}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Integer", " ", 
         RowBox[{"{", 
          RowBox[{"beficol", ",", "afticol", ",", "count"}], 
          "\[IndentingNewLine]", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"count", "=", "1"}], ",", 
         RowBox[{"count", "\[LessEqual]", "iterations"}], ",", 
         RowBox[{"count", "=", 
          RowBox[{"count", "+", "1"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"a", "=", "ain"}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
            "The", " ", "arguments", " ", "are", " ", "always", " ", 
             "generated", " ", "as", " ", "const", " ", "references", " ", 
             "and", " ", "therefore", " ", "cannot", " ", "be", " ", 
             RowBox[{"changed", ".", "\[IndentingNewLine]", "Actuall"}], " ", 
             "we", " ", "get", " ", "waste", " ", "of", " ", "space", " ", 
             "and", " ", "time", " ", "here", " ", "when", " ", "they", " ", 
             "are", " ", 
             RowBox[{"copied", "."}]}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{"b", "=", "bin"}], ";", "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"k", "=", "1"}], ",", 
             RowBox[{"k", "\[LessEqual]", "n"}], ",", 
             RowBox[{"k", "=", 
              RowBox[{"k", "+", "1"}]}], ",", 
             RowBox[{
              RowBox[{"ipiv", "[", 
               RowBox[{"[", "k", "]"}], "]"}], "=", "0"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"i", "=", "1"}], ",", 
             RowBox[{"i", "\[LessEqual]", "n"}], ",", 
             RowBox[{"i", "=", 
              RowBox[{"i", "+", "1"}]}], ",", "\[IndentingNewLine]", 
             RowBox[{"(*", 
              RowBox[{
              "Algorithm", " ", "first", " ", "finds", " ", "absolutely", " ",
                "largest", " ", "matrix", " ", "element"}], "*)"}], 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"amax", "=", "0.0"}], ";", "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"k", "=", "1"}], ",", 
                RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                RowBox[{"k", "=", 
                 RowBox[{"k", "+", "1"}]}], ",", "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"ipiv", "[", 
                    RowBox[{"[", "k", "]"}], "]"}], "\[Equal]", "0"}], ",", 
                  RowBox[{"For", "[", 
                   RowBox[{
                    RowBox[{"l", "=", "1"}], ",", 
                    RowBox[{"l", "\[LessEqual]", "n"}], ",", 
                    RowBox[{"l", "=", 
                    RowBox[{"l", "+", "1"}]}], ",", "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"ipiv", "[", 
                    RowBox[{"[", "l", "]"}], "]"}], "\[Equal]", "0"}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Abs", "[", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "l"}], "]"}], "]"}], "]"}], ">", 
                    "amax"}], ",", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"amax", "=", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "l"}], "]"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"irow", "=", "k"}], ";", "\[IndentingNewLine]", 
                    RowBox[{"icol", "=", "l"}]}]}], "]"}]}], "]"}]}], "]"}]}],
                  "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{
                "Abs", " ", "is", " ", "generated", " ", "as", " ", 
                 "LIGHTABS"}], ",", 
                RowBox[{"a", " ", "macro", " ", "in", " ", 
                 RowBox[{"lightmat", ".", "h"}]}]}], "*)"}], 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"ipiv", "[", 
                RowBox[{"[", "icol", "]"}], "]"}], "=", 
               RowBox[{
                RowBox[{"ipiv", "[", 
                 RowBox[{"[", "icol", "]"}], "]"}], "+", "1"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"ipiv", "[", 
                  RowBox[{"[", "icol", "]"}], "]"}], ">", "1"}], ",", 
                RowBox[{
                 RowBox[{
                 "\"\<*** Gauss2 input data error ***\>\"", ">>", 
                  "\"\<\>\""}], ";", "\[IndentingNewLine]", "Break"}]}], 
               "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"irow", "\[NotEqual]", "icol"}], ",", 
                RowBox[{
                 RowBox[{"For", "[", 
                  RowBox[{
                   RowBox[{"k", "=", "1"}], ",", 
                   RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                   RowBox[{"k", "=", 
                    RowBox[{"k", "+", "1"}]}], ",", 
                   RowBox[{
                    RowBox[{"tmp", "=", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"irow", ",", "k"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"irow", ",", "k"}], "]"}], "]"}], "=", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}], "=", 
                    "tmp"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
                 RowBox[{"For", "[", 
                  RowBox[{
                   RowBox[{"k", "=", "1"}], ",", 
                   RowBox[{"k", "\[LessEqual]", "m"}], ",", 
                   RowBox[{"k", "=", 
                    RowBox[{"k", "+", "1"}]}], ",", 
                   RowBox[{
                    RowBox[{"tmp", "=", 
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"irow", ",", "k"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"irow", ",", "k"}], "]"}], "]"}], "=", 
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}], "=", 
                    "tmp"}]}]}], "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"indxr", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "=", "irow"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"indxc", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "=", "icol"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"icol", ",", "icol"}], "]"}], "]"}], "\[Equal]", 
                 "0"}], ",", 
                RowBox[{
                 RowBox[{
                 "Print", "[", "\"\<*** Gauss2 input data error 2 ***\>\"", 
                  "]"}], ";", "Break"}]}], "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"pivinv", "=", 
               RowBox[{"1.0", "/", 
                RowBox[{"a", "[", 
                 RowBox[{"[", 
                  RowBox[{"icol", ",", "icol"}], "]"}], "]"}]}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"a", "[", 
                RowBox[{"[", 
                 RowBox[{"icol", ",", "icol"}], "]"}], "]"}], "=", "1.0"}], 
              ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"a", "[", 
                RowBox[{"[", 
                 RowBox[{"icol", ",", "_"}], "]"}], "]"}], "=", 
               RowBox[{
                RowBox[{"a", "[", 
                 RowBox[{"[", 
                  RowBox[{"icol", ",", "_"}], "]"}], "]"}], "*", "pivinv"}]}],
               ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"b", "[", 
                RowBox[{"[", 
                 RowBox[{"icol", ",", "_"}], "]"}], "]"}], "=", 
               RowBox[{
                RowBox[{"b", "[", 
                 RowBox[{"[", 
                  RowBox[{"icol", ",", "_"}], "]"}], "]"}], "*", "pivinv"}]}],
               ";", "\[IndentingNewLine]", 
              RowBox[{"dumc", "=", 
               RowBox[{"a", "[", 
                RowBox[{"[", 
                 RowBox[{"_", ",", "icol"}], "]"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"k", "=", "1"}], ",", 
                RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                RowBox[{"k", "=", 
                 RowBox[{"k", "+", "1"}]}], ",", 
                RowBox[{
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", "icol"}], "]"}], "]"}], "=", "0"}]}], 
               "]"}], ";", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"This", " ", "cannot", " ", "be", " ", "simplified"}],
                 "..."}], "*)"}], 
              RowBox[{
               RowBox[{"a", "[", 
                RowBox[{"[", 
                 RowBox[{"icol", ",", "icol"}], "]"}], "]"}], "=", "pivinv"}],
               ";", "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"k", "=", "1"}], ",", 
                RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                RowBox[{"k", "=", 
                 RowBox[{"k", "+", "1"}]}], ",", "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"k", "\[NotEqual]", "icol"}], ",", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "_"}], "]"}], "]"}], "=", 
                    RowBox[{
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "_"}], "]"}], "]"}], "-", 
                    RowBox[{
                    RowBox[{"dumc", "[", 
                    RowBox[{"[", "k", "]"}], "]"}], "*", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "_"}], "]"}], "]"}]}]}]}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "_"}], "]"}], "]"}], "=", 
                    RowBox[{
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "_"}], "]"}], "]"}], "-", 
                    RowBox[{
                    RowBox[{"dumc", "[", 
                    RowBox[{"[", "k", "]"}], "]"}], "*", 
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "_"}], "]"}], "]"}]}]}]}]}]}], 
                 "]"}]}], "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
            "Extra", " ", "semicolons", " ", "above", " ", "are", " ", 
             "harmful"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"l", "=", "n"}], ",", 
             RowBox[{"l", "\[GreaterEqual]", "1"}], ",", 
             RowBox[{"l", "=", 
              RowBox[{"l", "-", "1"}]}], ",", "\[IndentingNewLine]", 
             RowBox[{"For", "[", 
              RowBox[{
               RowBox[{"k", "=", "1"}], ",", 
               RowBox[{"k", "\[LessEqual]", "n"}], ",", 
               RowBox[{"k", "=", 
                RowBox[{"k", "+", "1"}]}], ",", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"tmp", "=", 
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", 
                    RowBox[{"indxr", "[", 
                    RowBox[{"[", "l", "]"}], "]"}]}], "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", 
                    RowBox[{"indxr", "[", 
                    RowBox[{"[", "l", "]"}], "]"}]}], "]"}], "]"}], "=", 
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", 
                    RowBox[{"indxc", "[", 
                    RowBox[{"[", "l", "]"}], "]"}]}], "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", 
                    RowBox[{"indxc", "[", 
                    RowBox[{"[", "l", "]"}], "]"}]}], "]"}], "]"}], "=", 
                 "tmp"}]}]}], "]"}]}], "]"}]}], ")"}]}], "]"}], ";", 
       "\[IndentingNewLine]", "b"}]}], "]"}]}], ";"}], "\n", 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.510641091036875*^9, 3.5106411922992597`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
The GaussSolveForLoops function 

\
\>", "Subsection",
 CellChangeTimes->{{3.510641222563916*^9, 3.51064122368888*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"GaussSolveForLoops", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"Real", "[", 
         RowBox[{"n_", ",", "n_"}], "]"}], " ", "ain_"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Real", "[", 
         RowBox[{"n_", ",", "m_"}], "]"}], " ", "bin_"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Integer", " ", "iterations_"}]}], "]"}], "\[Rule]", 
     RowBox[{"Real", "[", 
      RowBox[{"n", ",", "m"}], "]"}]}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Real", "[", "n", "]"}], " ", "dumc"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Real", "[", 
          RowBox[{"n", ",", "n"}], "]"}], " ", "a"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Real", "[", 
          RowBox[{"n", ",", "m"}], "]"}], " ", "b"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Integer", "[", "n", "]"}], " ", 
         RowBox[{"{", 
          RowBox[{"ipiv", ",", "indxr", ",", "indxc"}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Integer", " ", 
         RowBox[{"{", 
          RowBox[{"i", ",", "k", ",", "l", ",", "irow", ",", "icol"}], 
          "}"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"Real", " ", 
         RowBox[{"{", 
          RowBox[{"pivinv", ",", "amax", ",", "tmp"}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Integer", " ", 
         RowBox[{"{", 
          RowBox[{"beficol", ",", "afticol", ",", "count"}], "}"}]}]}], "}"}],
       ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"count", "=", "1"}], ",", 
         RowBox[{"count", "\[LessEqual]", "iterations"}], ",", 
         RowBox[{"count", "=", 
          RowBox[{"count", "+", "1"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"a", "=", "ain"}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
            "The", " ", "arguments", " ", "are", " ", "always", " ", 
             "generated", " ", "as", " ", "const", " ", "references", " ", 
             "and", " ", "therefore", " ", "cannot", " ", "be", " ", 
             RowBox[{"changed", ".", "\[IndentingNewLine]", "Actuall"}], " ", 
             "we", " ", "get", " ", "waste", " ", "of", " ", "space", " ", 
             "and", " ", "time", " ", "here", " ", "when", " ", "they", " ", 
             "are", " ", 
             RowBox[{"copied", "."}]}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{"b", "=", "bin"}], ";", "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"k", "=", "1"}], ",", 
             RowBox[{"k", "\[LessEqual]", "n"}], ",", 
             RowBox[{"k", "=", 
              RowBox[{"k", "+", "1"}]}], ",", 
             RowBox[{
              RowBox[{"ipiv", "[", 
               RowBox[{"[", "k", "]"}], "]"}], "=", "0"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"i", "=", "1"}], ",", 
             RowBox[{"i", "\[LessEqual]", "n"}], ",", 
             RowBox[{"i", "=", 
              RowBox[{"i", "+", "1"}]}], ",", "\[IndentingNewLine]", 
             RowBox[{"(*", 
              RowBox[{
              "Algorithm", " ", "first", " ", "finds", " ", "absolutely", " ",
                "largest", " ", "matrix", " ", "element"}], "*)"}], 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"amax", "=", "0.0"}], ";", "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"k", "=", "1"}], ",", 
                RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                RowBox[{"k", "=", 
                 RowBox[{"k", "+", "1"}]}], ",", "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"ipiv", "[", 
                    RowBox[{"[", "k", "]"}], "]"}], "\[Equal]", "0"}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{"For", "[", 
                   RowBox[{
                    RowBox[{"l", "=", "1"}], ",", 
                    RowBox[{"l", "\[LessEqual]", "n"}], ",", 
                    RowBox[{"l", "=", 
                    RowBox[{"l", "+", "1"}]}], ",", "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"ipiv", "[", 
                    RowBox[{"[", "l", "]"}], "]"}], "\[Equal]", "0"}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Abs", "[", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "l"}], "]"}], "]"}], "]"}], ">", 
                    "amax"}], ",", 
                    RowBox[{
                    RowBox[{"amax", "=", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "l"}], "]"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"irow", "=", "k"}], ";", "\[IndentingNewLine]", 
                    RowBox[{"icol", "=", "l"}]}]}], "]"}]}], "]"}]}], "]"}]}],
                  "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"ipiv", "[", 
                RowBox[{"[", "icol", "]"}], "]"}], "=", 
               RowBox[{
                RowBox[{"ipiv", "[", 
                 RowBox[{"[", "icol", "]"}], "]"}], "+", "1"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"ipiv", "[", 
                  RowBox[{"[", "icol", "]"}], "]"}], ">", "1"}], ",", 
                RowBox[{
                 RowBox[{
                 "\"\<*** Gauss2 input data error ***\>\"", ">>", 
                  "\"\<\>\""}], ";", "\[IndentingNewLine]", "Break"}]}], 
               "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"irow", "\[NotEqual]", "icol"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"For", "[", 
                  RowBox[{
                   RowBox[{"k", "=", "1"}], ",", 
                   RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                   RowBox[{"k", "=", 
                    RowBox[{"k", "+", "1"}]}], ",", 
                   RowBox[{
                    RowBox[{"tmp", "=", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"irow", ",", "k"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"irow", ",", "k"}], "]"}], "]"}], "=", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}], "=", 
                    "tmp"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
                 RowBox[{"For", "[", 
                  RowBox[{
                   RowBox[{"k", "=", "1"}], ",", 
                   RowBox[{"k", "\[LessEqual]", "m"}], ",", 
                   RowBox[{"k", "=", 
                    RowBox[{"k", "+", "1"}]}], ",", 
                   RowBox[{
                    RowBox[{"tmp", "=", 
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"irow", ",", "k"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"irow", ",", "k"}], "]"}], "]"}], "=", 
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}], "=", 
                    "tmp"}]}]}], "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"indxr", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "=", "irow"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"indxc", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "=", "icol"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"icol", ",", "icol"}], "]"}], "]"}], "\[Equal]", 
                 "0"}], ",", 
                RowBox[{
                 RowBox[{
                 "Print", "[", "\"\<*** Gauss2 input data error 2 ***\>\"", 
                  "]"}], ";", "\[IndentingNewLine]", "Break"}]}], "]"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"pivinv", "=", 
               RowBox[{"1.0", "/", 
                RowBox[{"a", "[", 
                 RowBox[{"[", 
                  RowBox[{"icol", ",", "icol"}], "]"}], "]"}]}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"a", "[", 
                RowBox[{"[", 
                 RowBox[{"icol", ",", "icol"}], "]"}], "]"}], "=", "1.0"}], 
              ";", "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"k", "=", "1"}], ",", 
                RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                RowBox[{"k", "=", 
                 RowBox[{"k", "+", "1"}]}], ",", 
                RowBox[{
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"icol", ",", "k"}], "]"}], "]"}], "=", 
                 RowBox[{
                  RowBox[{"a", "[", 
                   RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}], "*", 
                  "pivinv"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"k", "=", "1"}], ",", 
                RowBox[{"k", "\[LessEqual]", "m"}], ",", 
                RowBox[{"k", "=", 
                 RowBox[{"k", "+", "1"}]}], ",", 
                RowBox[{
                 RowBox[{"b", "[", 
                  RowBox[{"[", 
                   RowBox[{"icol", ",", "k"}], "]"}], "]"}], "=", 
                 RowBox[{
                  RowBox[{"b", "[", 
                   RowBox[{"[", 
                    RowBox[{"icol", ",", "k"}], "]"}], "]"}], "*", 
                  "pivinv"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"k", "=", "1"}], ",", 
                RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                RowBox[{"k", "=", 
                 RowBox[{"k", "+", "1"}]}], ",", 
                RowBox[{
                 RowBox[{
                  RowBox[{"dumc", "[", 
                   RowBox[{"[", "k", "]"}], "]"}], "=", 
                  RowBox[{"a", "[", 
                   RowBox[{"[", 
                    RowBox[{"k", ",", "icol"}], "]"}], "]"}]}], ";", 
                 RowBox[{
                  RowBox[{"a", "[", 
                   RowBox[{"[", 
                    RowBox[{"k", ",", "icol"}], "]"}], "]"}], "=", "0"}]}]}], 
               "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"a", "[", 
                RowBox[{"[", 
                 RowBox[{"icol", ",", "icol"}], "]"}], "]"}], "=", "pivinv"}],
               ";", "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"k", "=", "1"}], ",", 
                RowBox[{"k", "\[LessEqual]", "n"}], ",", 
                RowBox[{"k", "=", 
                 RowBox[{"k", "+", "1"}]}], ",", "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"k", "\[NotEqual]", "icol"}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"For", "[", 
                    RowBox[{
                    RowBox[{"l", "=", "1"}], ",", 
                    RowBox[{"l", "\[LessEqual]", "n"}], ",", 
                    RowBox[{"l", "=", 
                    RowBox[{"l", "+", "1"}]}], ",", 
                    RowBox[{
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "l"}], "]"}], "]"}], "=", 
                    RowBox[{
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "l"}], "]"}], "]"}], "-", 
                    RowBox[{
                    RowBox[{"dumc", "[", 
                    RowBox[{"[", "k", "]"}], "]"}], "*", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "l"}], "]"}], "]"}]}]}]}]}], "]"}], 
                   ";", "\[IndentingNewLine]", 
                   RowBox[{"For", "[", 
                    RowBox[{
                    RowBox[{"l", "=", "1"}], ",", 
                    RowBox[{"l", "\[LessEqual]", "m"}], ",", 
                    RowBox[{"l", "=", 
                    RowBox[{"l", "+", "1"}]}], ",", 
                    RowBox[{
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "l"}], "]"}], "]"}], "=", 
                    RowBox[{
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"k", ",", "l"}], "]"}], "]"}], "-", 
                    RowBox[{
                    RowBox[{"dumc", "[", 
                    RowBox[{"[", "k", "]"}], "]"}], "*", 
                    RowBox[{"b", "[", 
                    RowBox[{"[", 
                    RowBox[{"icol", ",", "l"}], "]"}], "]"}]}]}]}]}], 
                    "]"}]}]}], "]"}]}], "]"}]}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"l", "=", "n"}], ",", 
             RowBox[{"l", "\[GreaterEqual]", "1"}], ",", 
             RowBox[{"l", "=", 
              RowBox[{"l", "-", "1"}]}], ",", "\[IndentingNewLine]", 
             RowBox[{"For", "[", 
              RowBox[{
               RowBox[{"k", "=", "1"}], ",", 
               RowBox[{"k", "\[LessEqual]", "n"}], ",", 
               RowBox[{"k", "=", 
                RowBox[{"k", "+", "1"}]}], ",", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"tmp", "=", 
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", 
                    RowBox[{"indxr", "[", 
                    RowBox[{"[", "l", "]"}], "]"}]}], "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", 
                    RowBox[{"indxr", "[", 
                    RowBox[{"[", "l", "]"}], "]"}]}], "]"}], "]"}], "=", 
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", 
                    RowBox[{"indxc", "[", 
                    RowBox[{"[", "l", "]"}], "]"}]}], "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"a", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", 
                    RowBox[{"indxc", "[", 
                    RowBox[{"[", "l", "]"}], "]"}]}], "]"}], "]"}], "=", 
                 "tmp"}]}]}], "]"}]}], "]"}]}], ")"}]}], "]"}], ";", 
       "\[IndentingNewLine]", "b"}]}], "]"}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.5106412265794125`*^9, 3.5106413206389027`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Compiled GaussSolveForLoops function, using Compile[] ", \
"Subsection"],

Cell["\<\
GaussSolveForLoopsC=
Compile[{{ain,_Real,2},{bin,_Real,2},{iterations,_Integer}},
Module[{
\tn=Dimensions[ain][[1]],
\tm=Dimensions[bin][[2]],
\tdumc=Table[0.0,{n}],
\ta=Table[0.0,{n},{n}],
\tb=Table[0.0,{n},{m}],
\tipiv=Table[0,{n}], 
    indxr=Table[0,{n}], 
    indxc=Table[0,{n}],
    i=0, k=0, l=0, irow=0, icol=0,
    pivinv=0.0, amax=0.0, tmp=0.0,
    beficol=0, afticol=0, count=0
  },
  
For[count=1,count<=iterations,count=count+1,(
   a=ain;  (* The arguments are always generated as const references
              and therefore cannot be changed.
              Actuall we get waste of space and time here when they are \
copied.   *)
   b=bin;
   For [k=1,k<=n, k=k+1,ipiv[[k]]=0];
   
   For [i=1,i<=n, i=i+1,
 
     (* Algorithm first finds absolutely largest matrix element *)
     amax=0.0;
      For [k=1,k<=n, k=k+1,
       If [ ipiv[[k]]==0 ,
        For [l=1,l<=n, l=l+1,
          If [ ipiv[[l]]==0 ,
            If [ Abs[a[[k,l]]] > amax,
                    amax= Abs[a[[k,l]]];
                    irow=k;
                    icol=l
              ]]]]];


     ipiv[[icol]]=ipiv[[icol]]+1;
     If[ipiv[[icol]]>1,
        \"*** Gauss2 input data error ***\">>\"\"; 
        Break];
     If[ irow!=icol ,
        For [k=1,k<=n, k=k+1,
               tmp=a[[irow,k]] ;     
               a[[irow,k]]=a[[icol,k]]; 
               a[[icol,k]]=tmp];
        For [k=1,k<=m, k=k+1,
               tmp=b[[irow,k]] ;          
               b[[irow,k]]=b[[icol,k]]; 
               b[[icol,k]]=tmp]
        ];
      indxr[[i]]=irow;
      indxc[[i]]=icol;
      If [ a[[icol,icol]]==0,
            Print[\"*** Gauss2 input data error 2 ***\"];
            Break];
      
      pivinv=1.0 / a[[icol,icol]];
      a[[icol,icol]]=1.0;

      For [k=1,k<=n, k=k+1,  
          a[[icol,k]]=a[[icol,k]]*pivinv];
      For [k=1,k<=m, k=k+1,  
          b[[icol,k]]=b[[icol,k]]*pivinv];
      For [k=1,k<=n, k=k+1,  
          dumc[[k]]=a[[k,icol]];a[[k,icol]]=0];

      
      a[[icol,icol]]= pivinv;

      For [k=1,k<=n, k=k+1,  
       If [ k != icol,
         For [l=1,l<=n, l=l+1,
              a[[k,l]]= a[[k,l]]- dumc[[k]]* a[[icol,l]]];  
             For [l=1,l<=m, l=l+1,
              b[[k,l]]= b[[k,l]]- dumc[[k]]* b[[icol,l]]] 
          ]]
   ];

For [l=n,l>=1, l=l-1,
    For [k=1,k<=n, k=k+1,
     tmp= a[[k,indxr[[l]]]];
     a[[k,indxr[[l]]]]=a[[k,indxc[[l]]]];
     a[[k,indxc[[l]]]]=tmp
        ]]
   )]; 
  b
]
]; \
\>", "Input"],

Cell["\<\
End[];
EndPackage[];\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Executing the interpreted version in Mathematica", "Subsection"],

Cell["Create two random matrices.", "Text"],

Cell["\<\
a=Table[Random[],{10},{10}];
b=Table[Random[],{10},{2}];\
\>", "Input",
 PageWidth->DirectedInfinity[1],
 ShowSpecialCharacters->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Run the ArraySlice version", "Subsection"],

Cell[TextData[{
 "Note: Too short loop time may be spent for reliable measurements, only 3- 4 \
seconds combined with a resolution of 1 second. Relevant ",
 StyleBox["factor",
  FontWeight->"Bold"],
 " for 1.5 GHz computer is 10."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"factor", "=", "15"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["\<\
loops=2*factor;
s=(c=GaussSolveArraySlice[a,b,loops];)//Timing;
meval=s[[1]]/loops;
Print[\"TIMING FOR NON-COMPILED VERSION= \",meval];
Dot[a,c] - b // MatrixForm\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"TIMING FOR NON-COMPILED VERSION= \"\>", "\[InvisibleSpace]", 
   "0.11979999999999999`"}],
  SequenceForm["TIMING FOR NON-COMPILED VERSION= ", 0.11979999999999999`],
  Editable->False]], "Print",
 CellChangeTimes->{3.510639779110108*^9, 3.5106413461693354`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"3.3306690738754696`*^-16", "5.551115123125783`*^-16"},
     {
      RowBox[{"-", "1.3808398868775384`*^-15"}], 
      RowBox[{"-", "1.0547118733938987`*^-15"}]},
     {
      RowBox[{"-", "1.3322676295501878`*^-15"}], 
      RowBox[{"-", "6.661338147750939`*^-16"}]},
     {
      RowBox[{"-", "6.661338147750939`*^-16"}], 
      RowBox[{"-", "3.3306690738754696`*^-16"}]},
     {
      RowBox[{"-", "8.049116928532385`*^-16"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]},
     {
      RowBox[{"-", "4.996003610813204`*^-16"}], "1.734723475976807`*^-16"},
     {
      RowBox[{"-", "4.440892098500626`*^-16"}], 
      RowBox[{"-", "3.885780586188048`*^-16"}]},
     {
      RowBox[{"-", "5.551115123125783`*^-16"}], 
      RowBox[{"-", "4.440892098500626`*^-16"}]},
     {
      RowBox[{"-", "6.106226635438361`*^-16"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]},
     {
      RowBox[{"-", "1.1171619185290638`*^-15"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.5106397791882305`*^9, 3.51064134618496*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Run the For-loop version", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
loops=2*factor;
s=(c=GaussSolveForLoops[a,b,loops];)//Timing;
mevalFor=s[[1]]/loops;
Print[\"TIMING FOR NON-COMPILED VERSION (FOR-LOOPS)=\",mevalFor];
Dot[a,c] - b // MatrixForm\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"TIMING FOR NON-COMPILED VERSION (FOR-LOOPS)=\"\>", 
   "\[InvisibleSpace]", "0.06093333333333332`"}],
  SequenceForm[
  "TIMING FOR NON-COMPILED VERSION (FOR-LOOPS)=", 0.06093333333333332],
  Editable->False]], "Print",
 CellChangeTimes->{3.5106397792976017`*^9, 3.510641348184896*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"3.3306690738754696`*^-16", "5.551115123125783`*^-16"},
     {
      RowBox[{"-", "1.3808398868775384`*^-15"}], 
      RowBox[{"-", "1.0547118733938987`*^-15"}]},
     {
      RowBox[{"-", "1.3322676295501878`*^-15"}], 
      RowBox[{"-", "6.661338147750939`*^-16"}]},
     {
      RowBox[{"-", "6.661338147750939`*^-16"}], 
      RowBox[{"-", "3.3306690738754696`*^-16"}]},
     {
      RowBox[{"-", "8.049116928532385`*^-16"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]},
     {
      RowBox[{"-", "4.996003610813204`*^-16"}], "1.734723475976807`*^-16"},
     {
      RowBox[{"-", "4.440892098500626`*^-16"}], 
      RowBox[{"-", "3.885780586188048`*^-16"}]},
     {
      RowBox[{"-", "5.551115123125783`*^-16"}], 
      RowBox[{"-", "4.440892098500626`*^-16"}]},
     {
      RowBox[{"-", "6.106226635438361`*^-16"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]},
     {
      RowBox[{"-", "1.1171619185290638`*^-15"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.5106397793444757`*^9, 3.5106413482005205`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Run the Compiled version", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
loops=200*factor;
s=(c=GaussSolveForLoopsC[a,b,loops];)//Timing;

mevalForC=s[[1]]/loops;
Print[\"TIMING FOR VERSION (FOR-LOOPS) with Compile[]=\",mevalForC];
Dot[a,c] - b // MatrixForm\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"TIMING FOR VERSION (FOR-LOOPS) with Compile[]=\"\>", 
   "\[InvisibleSpace]", "0.00027100000000000003`"}],
  SequenceForm[
  "TIMING FOR VERSION (FOR-LOOPS) with Compile[]=", 0.00027100000000000003`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5106397802819457`*^9, 3.510641349106742*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"3.3306690738754696`*^-16", "5.551115123125783`*^-16"},
     {
      RowBox[{"-", "1.3808398868775384`*^-15"}], 
      RowBox[{"-", "1.0547118733938987`*^-15"}]},
     {
      RowBox[{"-", "1.3322676295501878`*^-15"}], 
      RowBox[{"-", "6.661338147750939`*^-16"}]},
     {
      RowBox[{"-", "6.661338147750939`*^-16"}], 
      RowBox[{"-", "3.3306690738754696`*^-16"}]},
     {
      RowBox[{"-", "8.049116928532385`*^-16"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]},
     {
      RowBox[{"-", "4.996003610813204`*^-16"}], "1.734723475976807`*^-16"},
     {
      RowBox[{"-", "4.440892098500626`*^-16"}], 
      RowBox[{"-", "3.885780586188048`*^-16"}]},
     {
      RowBox[{"-", "5.551115123125783`*^-16"}], 
      RowBox[{"-", "4.440892098500626`*^-16"}]},
     {
      RowBox[{"-", "6.106226635438361`*^-16"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]},
     {
      RowBox[{"-", "1.1171619185290638`*^-15"}], 
      RowBox[{"-", "1.1102230246251565`*^-16"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.51063978029757*^9, 3.510641349106742*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Run the built-in Mathematica function LinearSolve[]", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
loops=2000*factor;
s=(Do[c=LinearSolve[a,b],{loops}];)//Timing;
internalEval=s[[1]]/loops\
\>", "Input"],

Cell[BoxData["0.00002653333333333334`"], "Output",
 CellChangeTimes->{3.5106397812350397`*^9, 3.5106413499192157`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generation of Fortran90 Code", "Section"],

Cell["\<\
The command CompilePackage translates the package to Fortran90 code:\
\>", "Text"],

Cell[CellGroupData[{

Cell["CompilePackage[\"Gauss\"]", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData["\<\"Successful compilation to Fortran90: 2 function(s)\"\>"], \
"Print",
 CellChangeTimes->{3.5106397835787153`*^9, 3.510641352528507*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Building the Executable", "Section"],

Cell["\<\
Call MakeBinary to compile the generated Fortran90 code and build the \
executable(s).\
\>", "Text"],

Cell["MakeBinary[]", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Installing Compiled Code", "Section"],

Cell["\<\
Interpreted versions are removed, and compiled ones are used instead:\
\>", "Text"],

Cell[CellGroupData[{

Cell["InstallCode[\"Gauss\"]", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Gauss\"\>", "\[InvisibleSpace]", "\<\" is installed.\"\>"}],
  SequenceForm["Gauss", " is installed."],
  Editable->False]], "Print",
 CellChangeTimes->{3.510639788625428*^9, 3.51064135993452*^9}],

Cell[BoxData[
 RowBox[{"LinkObject", "[", 
  RowBox[{"\<\"\\\"C:\\\\b\\\\omath\\\\MathCode\\\\DemosF90\\\\Gauss\\\\\
Gaussml.exe\\\"\"\>", ",", "17", ",", "8"}], "]"}]], "Output",
 CellChangeTimes->{3.510639788625428*^9, 3.51064135993452*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Test", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"c", "=", 
   RowBox[{"GaussSolveArraySlice", "[", 
    RowBox[{"a", ",", "b", ",", "1"}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Dot", "[", 
    RowBox[{"a", ",", "c"}], "]"}], "-", "b"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1.4432899320127035`*^-15"}], 
      RowBox[{"-", "1.2212453270876722`*^-15"}]},
     {
      RowBox[{"-", "1.824929096727601`*^-15"}], 
      RowBox[{"-", "1.8318679906315083`*^-15"}]},
     {
      RowBox[{"-", "2.220446049250313`*^-15"}], 
      RowBox[{"-", "1.6653345369377348`*^-15"}]},
     {
      RowBox[{"-", "5.551115123125783`*^-16"}], 
      RowBox[{"-", "3.3306690738754696`*^-16"}]},
     {
      RowBox[{"-", "9.159339953157541`*^-16"}], 
      RowBox[{"-", "6.661338147750939`*^-16"}]},
     {
      RowBox[{"-", "1.3877787807814457`*^-15"}], 
      RowBox[{"-", "9.645062526431047`*^-16"}]},
     {
      RowBox[{"-", "8.881784197001252`*^-16"}], 
      RowBox[{"-", "1.0547118733938987`*^-15"}]},
     {
      RowBox[{"-", "1.1102230246251565`*^-15"}], 
      RowBox[{"-", "1.1379786002407855`*^-15"}]},
     {
      RowBox[{"-", "1.609823385706477`*^-15"}], 
      RowBox[{"-", "1.4432899320127035`*^-15"}]},
     {
      RowBox[{"-", "1.9186041644303486`*^-15"}], 
      RowBox[{"-", "1.5543122344752192`*^-15"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.5106397886879263`*^9, 3.510641359965769*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"c", "=", 
   RowBox[{"GaussSolveForLoops", "[", 
    RowBox[{"a", ",", "b", ",", "1"}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Dot", "[", 
    RowBox[{"a", ",", "c"}], "]"}], "-", "b"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1.4432899320127035`*^-15"}], 
      RowBox[{"-", "1.2212453270876722`*^-15"}]},
     {
      RowBox[{"-", "1.824929096727601`*^-15"}], 
      RowBox[{"-", "1.8318679906315083`*^-15"}]},
     {
      RowBox[{"-", "2.220446049250313`*^-15"}], 
      RowBox[{"-", "1.6653345369377348`*^-15"}]},
     {
      RowBox[{"-", "5.551115123125783`*^-16"}], 
      RowBox[{"-", "3.3306690738754696`*^-16"}]},
     {
      RowBox[{"-", "9.159339953157541`*^-16"}], 
      RowBox[{"-", "6.661338147750939`*^-16"}]},
     {
      RowBox[{"-", "1.3877787807814457`*^-15"}], 
      RowBox[{"-", "9.645062526431047`*^-16"}]},
     {
      RowBox[{"-", "8.881784197001252`*^-16"}], 
      RowBox[{"-", "1.0547118733938987`*^-15"}]},
     {
      RowBox[{"-", "1.1102230246251565`*^-15"}], 
      RowBox[{"-", "1.1379786002407855`*^-15"}]},
     {
      RowBox[{"-", "1.609823385706477`*^-15"}], 
      RowBox[{"-", "1.4432899320127035`*^-15"}]},
     {
      RowBox[{"-", "1.9186041644303486`*^-15"}], 
      RowBox[{"-", "1.5543122344752192`*^-15"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.5106397888129225`*^9, 3.510641359997018*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["External Execution", "Section"],

Cell[CellGroupData[{

Cell["Preperations", "Subsection"],

Cell["\<\
SetAttributes[AbsTime,HoldFirst];
AbsTime[x_] := Module[{start,res},
\t\tstart = AbsoluteTime[];
\t\tres=x;
\t\t{(AbsoluteTime[]-start) Second,res}
\t];\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[CellGroupData[{

Cell["\<\
loops=15000*factor;
externaleval=((c=GaussSolveArraySlice[a,b,loops];  )//AbsTime)[[1]]/loops\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 RowBox[{"0.00013707894666666666666653295`8.940698270199917", " ", 
  "Second"}]], "Output",
 CellChangeTimes->{3.510639788891045*^9, 3.5106413908866544`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
loops=200*factor;
externalevalPass=((Do[c=GaussSolveArraySlice[a,b,1],{loops}];)//AbsTime)[[1]]/\
loops\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 RowBox[{"0.00023957566666666666666101378`7.308108953547872", " ", 
  "Second"}]], "Output",
 CellChangeTimes->{3.5106397889222937`*^9, 3.510641391652255*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
loops=15000*factor;
externalevalFor=((c=GaussSolveForLoops[a,b,loops];  )//AbsTime)[[1]]/loops\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 RowBox[{"0.00003048513555555555555545429`8.287815642108422", " ", 
  "Second"}]], "Output",
 CellChangeTimes->{3.5106397889379187`*^9, 3.5106413985270348`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
loops=200*factor;
externalevalPassFor=((Do[c=GaussSolveForLoops[a,b,1],{loops}];)//AbsTime)[[1]]\
/loops\
\>", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 RowBox[{"0.00013020416666666666665573791`7.043291130538338", " ", 
  "Second"}]], "Output",
 CellChangeTimes->{3.5106397889535427`*^9, 3.510641398964521*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dot[a,c] - b // MatrixForm", "Input",
 PageWidth->Infinity,
 ShowSpecialCharacters->False],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1.4432899320127035`*^-15"}], 
      RowBox[{"-", "1.2212453270876722`*^-15"}]},
     {
      RowBox[{"-", "1.824929096727601`*^-15"}], 
      RowBox[{"-", "1.8318679906315083`*^-15"}]},
     {
      RowBox[{"-", "2.220446049250313`*^-15"}], 
      RowBox[{"-", "1.6653345369377348`*^-15"}]},
     {
      RowBox[{"-", "5.551115123125783`*^-16"}], 
      RowBox[{"-", "3.3306690738754696`*^-16"}]},
     {
      RowBox[{"-", "9.159339953157541`*^-16"}], 
      RowBox[{"-", "6.661338147750939`*^-16"}]},
     {
      RowBox[{"-", "1.3877787807814457`*^-15"}], 
      RowBox[{"-", "9.645062526431047`*^-16"}]},
     {
      RowBox[{"-", "8.881784197001252`*^-16"}], 
      RowBox[{"-", "1.0547118733938987`*^-15"}]},
     {
      RowBox[{"-", "1.1102230246251565`*^-15"}], 
      RowBox[{"-", "1.1379786002407855`*^-15"}]},
     {
      RowBox[{"-", "1.609823385706477`*^-15"}], 
      RowBox[{"-", "1.4432899320127035`*^-15"}]},
     {
      RowBox[{"-", "1.9186041644303486`*^-15"}], 
      RowBox[{"-", "1.5543122344752192`*^-15"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.510639789016041*^9, 3.5106413989801455`*^9}]
}, Open  ]],

Cell["UninstallCode[\"Gauss\"];", "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Performance Comparison", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{
   RowBox[{"(", GridBox[{
      {"meval", 
       RowBox[{"meval", "/", "externaleval"}]},
      {"mevalFor", 
       RowBox[{"mevalFor", "/", "externaleval"}]},
      {"mevalForC", 
       RowBox[{"mevalForC", "/", "externaleval"}]},
      {"externalevalPass", 
       RowBox[{"externalevalPass", "/", "externaleval"}]},
      {"externalevalPassFor", 
       RowBox[{"externalevalPassFor", "/", "externaleval"}]},
      {"externaleval", 
       RowBox[{"externaleval", "/", "externaleval"}]},
      {"externalevalFor", 
       RowBox[{"externalevalFor", "/", "externaleval"}]},
      {"internalEval", 
       RowBox[{"internalEval", "/", "externaleval"}]}
     }], ")"}], " ", ",", "\[IndentingNewLine]", 
   RowBox[{"TableHeadings", "->", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "\"\<GaussArraySlice, Standard Mathematica\>\"", ",", 
        "\[IndentingNewLine]", "\"\<- GaussForLoop, Standard Mathematica\>\"",
         ",", "\[IndentingNewLine]", "\"\<- GaussForLoop, Compile[]\>\"", ",",
         "\[IndentingNewLine]", 
        "\"\<GaussArraySlice, F90 via MathLink + call overhead\>\"", ",", 
        "\[IndentingNewLine]", 
        "\"\<- GaussForLoop F90 via MathLink + call overhead\>\"", ",", 
        "\[IndentingNewLine]", "\"\<GaussArraySlice, F90\>\"", ",", 
        "\[IndentingNewLine]", "\"\<- GaussForLoop F90\>\"", ",", 
        "\[IndentingNewLine]", " ", "\"\<LinearSolve[]\>\""}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"\"\<Time consumed\>\"", ",", "\"\<Relative\>\""}], "}"}]}], 
     "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      StyleBox["\[Null]",
       ShowStringCharacters->False], 
      TagBox["\<\"Time consumed\"\>",
       HoldForm], 
      TagBox["\<\"Relative\"\>",
       HoldForm]},
     {
      TagBox["\<\"GaussArraySlice, Standard Mathematica\"\>",
       HoldForm], "0.11979999999999999`", 
      FractionBox["873.9489390104252`", "Second"]},
     {
      TagBox["\<\"- GaussForLoop, Standard Mathematica\"\>",
       HoldForm], "0.06093333333333332`", 
      FractionBox["444.5127046497098`", "Second"]},
     {
      TagBox["\<\"- GaussForLoop, Compile[]\"\>",
       HoldForm], "0.00027100000000000003`", 
      FractionBox["1.976962958863316`", "Second"]},
     {
      TagBox["\<\"GaussArraySlice, F90 via MathLink + call overhead\"\>",
       HoldForm], 
      RowBox[{
      "0.00023957566666666666666101378`7.308108953547872", " ", "Second"}], 
      "1.7477203647416413373`7.298104732434866"},
     {
      TagBox["\<\"- GaussForLoop F90 via MathLink + call overhead\"\>",
       HoldForm], 
      RowBox[{
      "0.00013020416666666666665573791`7.043291130538338", " ", "Second"}], 
      "0.9498480243161094224`7.0378254847538395"},
     {
      TagBox["\<\"GaussArraySlice, F90\"\>",
       HoldForm], 
      RowBox[{
      "0.00013707894666666666666653295`8.940698270199917", " ", "Second"}], 
      "1.`8.639668274535937"},
     {
      TagBox["\<\"- GaussForLoop F90\"\>",
       HoldForm], 
      RowBox[{
      "0.00003048513555555555555545429`8.287815642108422", " ", "Second"}], 
      "0.2223910840932117528`8.200605468533253"},
     {
      TagBox["\<\"LinearSolve[]\"\>",
       HoldForm], "0.00002653333333333334`", 
      FractionBox["0.19356242500063958`", "Second"]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxDividers->{
     "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
      "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[2.0999999999999996`]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   {OutputFormsDump`HeadedRows, OutputFormsDump`HeadedColumns}],
  Function[BoxForm`e$, 
   TableForm[
   BoxForm`e$, 
    TableHeadings -> {{
      "GaussArraySlice, Standard Mathematica", 
       "- GaussForLoop, Standard Mathematica", "- GaussForLoop, Compile[]", 
       "GaussArraySlice, F90 via MathLink + call overhead", 
       "- GaussForLoop F90 via MathLink + call overhead", 
       "GaussArraySlice, F90", "- GaussForLoop F90", "LinearSolve[]"}, {
      "Time consumed", "Relative"}}]]]], "Output",
 CellChangeTimes->{3.510639789297282*^9, 3.510641399027019*^9}]
}, Open  ]],

Cell["\<\
These values were obtained on a 1.5 GHz computer with  MathCode 1.2 /F90 and \
Mathematica 5 installed.\
\>", "Text"],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"\"\>", "\<\"Time consumed\"\>", "\<\"Relative\"\>"},
    {"\<\"GaussArraySlice, Standard Mathematica\"\>", 
     RowBox[{"0.2827333333333333`", " ", "Second"}], "16672.841501937994`"},
    {"\<\"- GaussForLoop, Standard Mathematica\"\>", 
     RowBox[{"0.18626666666666664`", " ", "Second"}], "10984.182776801405`"},
    {"\<\"- GaussForLoop, Compile[]\"\>", 
     RowBox[{"0.001849333333333334`", " ", "Second"}], "109.05555842107054`"},
    {"\<\"GaussArraySlice, F90 via MathLink + call overhead\"\>", 
     RowBox[{
     "0.0021097002666666666666582299`8.25288700598758", " ", "Second"}], 
     "124.4094488188976377964`7.828202536302919"},
    {"\<\"- GaussForLoop F90 via MathLink + call overhead\"\>", 
     RowBox[{
     "0.0021197146666666666666562456`8.254943652997168", " ", "Second"}], 
     "125.0000000000000000011`7.828974920724889"},
    {"\<\"GaussArraySlice, F90\"\>", 
     RowBox[{
     "0.0000169577173333333333331054`8.033094903380814", " ", "Second"}], 
     "1.`7.732064907716832"},
    {"\<\"- GaussForLoop F90\"\>", 
     RowBox[{
     "0.0000194501902222222222221429`8.092651364675616", " ", "Second"}], 
     "1.1469816272965879265`7.760823036679913"},
    {"\<\"LinearSolve[]\"\>", 
     RowBox[{
     "0.0000864463999999999999990577`7.865413160311535", " ", "Second"}], 
     "5.0977615855215733438`7.640181038698123"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[
   BoxForm`e$, 
    TableHeadings -> {{
      "GaussArraySlice, Standard Mathematica", 
       "- GaussForLoop, Standard Mathematica", "- GaussForLoop, Compile[]", 
       "GaussArraySlice, F90 via MathLink + call overhead", 
       "- GaussForLoop F90 via MathLink + call overhead", 
       "GaussArraySlice, F90", "- GaussForLoop F90", "LinearSolve[]"}, {
      "Time consumed", "Relative"}}]]]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Clean Up", "Section"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"UninstallCode", "[", "\"\<Gauss\>\"", "]"}], ";"}], "\n", 
   RowBox[{"CleanMathCodeFiles", "[", 
    RowBox[{
     RowBox[{"Confirm", "\[Rule]", "False"}], ",", 
     RowBox[{"CleanAllBut", "\[Rule]", 
      RowBox[{"{", "}"}]}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5106415270697966`*^9, 3.5106415325383716`*^9}}]
}, Open  ]]
},
ScreenStyleEnvironment->"Working",
WindowToolbars->{},
WindowSize->{1272, 922},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
FrontEndVersion->"8.0 for Microsoft Windows (32-bit) (November 7, 2010)",
StyleDefinitions->Notebook[{
   Cell[
    CellGroupData[{
      Cell["Style Definitions", "Subtitle"], 
      Cell[
      "Modify the definitions below to change the default appearance of all \
cells in a given style.  Make modifications to any definition using commands \
in the Format menu.", "Text"], 
      Cell[
       CellGroupData[{
         Cell["Style Environment Names", "Section"], 
         Cell[
          StyleData[All, "Working"], PageWidth -> WindowWidth, ScriptMinSize -> 
          9], 
         Cell[
          StyleData[All, "Presentation"], PageWidth -> WindowWidth, 
          ScriptMinSize -> 12, FontSize -> 16], 
         Cell[
          StyleData[All, "Condensed"], PageWidth -> WindowWidth, 
          CellBracketOptions -> {"Margins" -> {1, 1}, "Widths" -> {0, 5}}, 
          ScriptMinSize -> 8, FontSize -> 11], 
         Cell[
          StyleData[All, "Printout"], PageWidth -> PaperWidth, ScriptMinSize -> 
          7, FontSize -> 10, 
          PrivateFontOptions -> {"FontType" -> "Outline"}]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Notebook Options", "Section"], 
         Cell[
         "The options defined for the style below will be used at the \
Notebook level.", "Text"], 
         Cell[
          StyleData["Notebook"], PageHeaders -> {{
             Cell[
              TextData[{
                CounterBox["Page"]}], "PageNumber"], None, 
             Cell[
              TextData[{
                ValueBox["FileName"]}], "Header"]}, {
             Cell[
              TextData[{
                ValueBox["FileName"]}], "Header"], None, 
             Cell[
              TextData[{
                CounterBox["Page"]}], "PageNumber"]}}, 
          PageHeaderLines -> {True, True}, 
          PrintingOptions -> {
           "FacingPages" -> True, "FirstPageHeader" -> False}, 
          CellFrameLabelMargins -> 6, StyleMenuListing -> None]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Headings", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Title"], ShowCellBracket -> False, 
             CellMargins -> {{0, 0}, {0, 0}}, PageBreakBelow -> False, 
             LineSpacing -> {1, 0}, CounterIncrements -> "Title", 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}, {"Subtitle", 0}, {"Subsubtitle", 0}}, FontSize -> 
             34, FontColor -> GrayLevel[1], Background -> RGBColor[0, 0, 1]], 
            Cell[
             StyleData["Title", "Presentation"], 
             CellMargins -> {{0, 0}, {0, 0}}, LineSpacing -> {1, 0}, FontSize -> 
             44], 
            Cell[
             StyleData["Title", "Condensed"], CellMargins -> {{0, 0}, {0, 0}},
              FontSize -> 20], 
            Cell[
             StyleData["Title", "Printout"], CellMargins -> {{0, 0}, {0, 0}}, 
             FontSize -> 24, FontTracking -> "Plain", Background -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subtitle"], ShowCellBracket -> False, 
             CellMargins -> {{0, 0}, {0, 0}}, PageBreakBelow -> False, 
             LineSpacing -> {1, 3}, ParagraphIndent -> -96, CounterIncrements -> 
             "Subtitle", 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}, {"Subsubtitle", 0}}, FontFamily -> "Helvetica", 
             FontSize -> 18, FontColor -> GrayLevel[1], Background -> 
             RGBColor[0.2, 0.700008, 0.700008]], 
            Cell[
             StyleData["Subtitle", "Presentation"], 
             CellMargins -> {{0, 0}, {0, 0}}, LineSpacing -> {1, 6}, 
             ParagraphIndent -> -157, FontSize -> 30], 
            Cell[
             StyleData["Subtitle", "Condensed"], 
             CellMargins -> {{0, 0}, {0, 0}}, ParagraphIndent -> -78, 
             FontSize -> 14], 
            Cell[
             StyleData["Subtitle", "Printout"], 
             CellMargins -> {{0, 0}, {0, 0}}, ParagraphIndent -> -85, 
             FontSize -> 16, Background -> GrayLevel[0.6]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsubtitle"], ShowCellBracket -> False, 
             CellMargins -> {{10, 4}, {30, 10}}, PageBreakBelow -> False, 
             CounterIncrements -> "Subsubtitle", 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}}, FontFamily -> "Helvetica", FontSize -> 14, 
             FontSlant -> "Italic"], 
            Cell[
             StyleData["Subsubtitle", "Presentation"], 
             CellMargins -> {{8, 10}, {40, 20}}, LineSpacing -> {1, 0}, 
             FontSize -> 24], 
            Cell[
             StyleData["Subsubtitle", "Condensed"], 
             CellMargins -> {{8, 10}, {12, 8}}, FontSize -> 12], 
            Cell[
             StyleData["Subsubtitle", "Printout"], 
             CellMargins -> {{9, 10}, {50, 10}}, FontSize -> 14]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, CellDingbat -> 
             None, CellMargins -> {{12, Inherited}, {4, 24}}, 
             CellGroupingRules -> {"SectionGrouping", 30}, PageBreakBelow -> 
             False, CellFrameMargins -> 6, CounterIncrements -> "Section", 
             CounterAssignments -> {{"Subsection", 0}, {"Subsubsection", 0}}, 
             FontFamily -> "Helvetica", FontSize -> 16, FontWeight -> "Bold", 
             FontColor -> RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["Section", "Presentation"], 
             CellMargins -> {{10, 10}, {8, 32}}, LineSpacing -> {1, 2}, 
             FontSize -> 24, FontTracking -> "Condensed"], 
            Cell[
             StyleData["Section", "Condensed"], 
             CellMargins -> {{8, Inherited}, {2, 12}}, FontSize -> 12], 
            Cell[
             StyleData["Section", "Printout"], 
             CellMargins -> {{9, 0}, {2, 50}}, FontSize -> 14, FontTracking -> 
             "Plain", FontColor -> GrayLevel[0]]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsection"], 
             CellMargins -> {{12, Inherited}, {8, 20}}, 
             CellGroupingRules -> {"SectionGrouping", 40}, PageBreakBelow -> 
             False, CounterIncrements -> "Subsection", 
             CounterAssignments -> {{"Subsubsection", 0}}, FontFamily -> 
             "Helvetica", FontSize -> 13, FontWeight -> "Bold"], 
            Cell[
             StyleData["Subsection", "Presentation"], 
             CellMargins -> {{11, 10}, {8, 32}}, LineSpacing -> {1, 0}, 
             FontSize -> 22], 
            Cell[
             StyleData["Subsection", "Condensed"], 
             CellMargins -> {{8, Inherited}, {2, 12}}, FontSize -> 12], 
            Cell[
             StyleData["Subsection", "Printout"], 
             CellMargins -> {{9, 0}, {4, 40}}, FontSize -> 12]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsubsection"], CellDingbat -> "\[FilledSquare]", 
             CellMargins -> {{25, Inherited}, {8, 12}}, 
             CellGroupingRules -> {"SectionGrouping", 50}, PageBreakBelow -> 
             False, CounterIncrements -> "Subsubsection", FontFamily -> 
             "Times", FontSize -> 13, FontWeight -> "Bold"], 
            Cell[
             StyleData["Subsubsection", "Presentation"], 
             CellMargins -> {{29, 10}, {8, 26}}, LineSpacing -> {1, 0}, 
             FontSize -> 18], 
            Cell[
             StyleData["Subsubsection", "Condensed"], 
             CellMargins -> {{22, Inherited}, {2, 12}}, FontSize -> 10], 
            Cell[
             StyleData["Subsubsection", "Printout"], 
             CellMargins -> {{21, 0}, {4, 20}}, FontSize -> 11]}, Open]]}, 
        Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Body Text", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Text"], CellMargins -> {{12, 10}, {5, 5}}, 
             LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 12}, 
             CounterIncrements -> "Text", FontFamily -> "Times"], 
            Cell[
             StyleData["Text", "Presentation"], 
             CellMargins -> {{13, 10}, {8, 8}}, LineSpacing -> {1, 5}, 
             ParagraphSpacing -> {0, 12}], 
            Cell[
             StyleData["Text", "Condensed"], CellMargins -> {{8, 10}, {4, 4}},
              LineSpacing -> {1, 1}, ParagraphSpacing -> {0, 4}], 
            Cell[
             StyleData["Text", "Printout"], CellMargins -> {{9, 0}, {4, 4}}, 
             ParagraphSpacing -> {0, 6}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SmallText"], CellMargins -> {{12, 10}, {5, 5}}, 
             LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 6}, 
             CounterIncrements -> "SmallText", FontFamily -> "Helvetica", 
             FontSize -> 9], 
            Cell[
             StyleData["SmallText", "Presentation"], 
             CellMargins -> {{13, 10}, {8, 8}}, LineSpacing -> {1, 5}, 
             FontSize -> 12], 
            Cell[
             StyleData["SmallText", "Condensed"], 
             CellMargins -> {{8, 10}, {2, 2}}, LineSpacing -> {1, 2}, 
             FontSize -> 9], 
            Cell[
             StyleData["SmallText", "Printout"], 
             CellMargins -> {{9, 0}, {4, 4}}, FontSize -> 7]}, Closed]]}, 
        Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Input/Output", "Section"], 
         Cell[
         "The cells in this section define styles used for input and output \
to the kernel.  Be careful when modifying, renaming, or removing these \
styles, because the front end associates special meanings with these style \
names.", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Input"], CellFrame -> {{3, 0}, {0, 0}}, 
             CellMargins -> {{52, 10}, {8, 8}}, Evaluatable -> True, 
             CellGroupingRules -> "InputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             CellLabelMargins -> {{5, Inherited}, {Inherited, Inherited}}, 
             DefaultFormatType -> DefaultInputFormatType, FormatType -> 
             InputForm, ShowStringCharacters -> True, NumberMarks -> True, 
             CounterIncrements -> "Input", FontWeight -> "Bold", Background -> 
             RGBColor[1, 1, 0]], 
            Cell[
             StyleData["Input", "Presentation"], 
             CellMargins -> {{62, Inherited}, {10, 10}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Input", "Condensed"], 
             CellMargins -> {{40, 10}, {4, 4}}], 
            Cell[
             StyleData["Input", "Printout"], CellMargins -> {{44, 0}, {6, 6}},
              Background -> GrayLevel[0.8]]}, Closed]], 
         Cell[
          StyleData["InlineInput"], Evaluatable -> True, CellGroupingRules -> 
          "InputGrouping", CellHorizontalScrolling -> True, PageBreakWithin -> 
          False, GroupPageBreakWithin -> False, DefaultFormatType -> 
          DefaultInputFormatType, AutoItalicWords -> {}, FormatType -> 
          InputForm, ShowStringCharacters -> True, NumberMarks -> True, 
          CounterIncrements -> "Input", FontWeight -> "Bold"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Output"], CellFrame -> {{3, 0}, {0, 0}}, 
             CellMargins -> {{52, 10}, {8, 8}}, CellEditDuplicate -> True, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, 
             CellLabelMargins -> {{3, Inherited}, {Inherited, Inherited}}, 
             DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, CounterIncrements -> "Output", Background -> 
             RGBColor[1, 0.900008, 0.900008]], 
            Cell[
             StyleData["Output", "Presentation"], 
             CellMargins -> {{62, Inherited}, {12, 5}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Output", "Condensed"], 
             CellMargins -> {{40, Inherited}, {4, 1}}], 
            Cell[
             StyleData["Output", "Printout"], 
             CellMargins -> {{44, 0}, {6, 2}}, Background -> 
             GrayLevel[0.900008]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Message"], 
             CellMargins -> {{62, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "OutputGrouping", PageBreakWithin -> False, 
             GroupPageBreakWithin -> False, GeneratedCell -> True, 
             CellAutoOverwrite -> True, ShowCellLabel -> False, 
             DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, CounterIncrements -> "Message", StyleMenuListing -> 
             None, FontColor -> RGBColor[1, 0, 0]], 
            Cell[
             StyleData["Message", "Presentation"], 
             CellMargins -> {{74, Inherited}, {Inherited, Inherited}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Message", "Condensed"], 
             CellMargins -> {{50, Inherited}, {Inherited, Inherited}}], 
            Cell[
             StyleData["Message", "Printout"], 
             CellMargins -> {{54, Inherited}, {Inherited, Inherited}}, 
             FontColor -> GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Print"], 
             CellMargins -> {{62, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, ShowCellLabel -> 
             False, DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, CounterIncrements -> "Print", StyleMenuListing -> 
             None], 
            Cell[
             StyleData["Print", "Presentation"], 
             CellMargins -> {{74, Inherited}, {Inherited, Inherited}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Print", "Condensed"], 
             CellMargins -> {{50, Inherited}, {Inherited, Inherited}}], 
            Cell[
             StyleData["Print", "Printout"], 
             CellMargins -> {{54, Inherited}, {Inherited, Inherited}}]}, 
           Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Graphics"], 
             CellMargins -> {{62, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "GraphicsGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GeneratedCell -> True, 
             CellAutoOverwrite -> True, ShowCellLabel -> False, 
             DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, CounterIncrements -> "Graphics", StyleMenuListing -> 
             None], 
            Cell[
             StyleData["Graphics", "Presentation"], 
             CellMargins -> {{74, Inherited}, {Inherited, Inherited}}], 
            Cell[
             StyleData["Graphics", "Condensed"], 
             CellMargins -> {{52, Inherited}, {Inherited, Inherited}}, 
             ImageSize -> {175, 175}], 
            Cell[
             StyleData["Graphics", "Printout"], 
             CellMargins -> {{54, Inherited}, {Inherited, Inherited}}, 
             ImageSize -> {250, 250}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["CellLabel"], StyleMenuListing -> None, FontFamily -> 
             "Helvetica", FontSize -> 11, FontWeight -> "Bold", FontColor -> 
             RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["CellLabel", "Presentation"], FontSize -> 12], 
            Cell[
             StyleData["CellLabel", "Condensed"], FontSize -> 8], 
            Cell[
             StyleData["CellLabel", "Printout"], FontSize -> 8, FontColor -> 
             GrayLevel[0]]}, Closed]]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Unique Styles", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Author"], ShowCellBracket -> False, 
             CellMargins -> {{10, 4}, {2, 10}}, FontSize -> 16, FontSlant -> 
             "Italic"], 
            Cell[
             StyleData["Author", "Presentation"], 
             CellMargins -> {{12, 10}, {2, 12}}, LineSpacing -> {1, 5}, 
             ParagraphSpacing -> {0, 12}, FontSize -> 20], 
            Cell[
             StyleData["Author", "Condensed"], 
             CellMargins -> {{8, 10}, {1, 4}}, LineSpacing -> {1, 1}, 
             ParagraphSpacing -> {0, 4}, FontSize -> 12], 
            Cell[
             StyleData["Author", "Printout"], 
             CellMargins -> {{9, 0}, {4, 12}}, ParagraphSpacing -> {0, 6}, 
             FontSize -> 14]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Copyright"], ShowCellBracket -> False, 
             CellMargins -> {{10, 10}, {40, 2}}, FontFamily -> "Helvetica", 
             FontSize -> 9], 
            Cell[
             StyleData["Copyright", "Presentation"], 
             CellMargins -> {{12, 10}, {50, 2}}, LineSpacing -> {1, 5}, 
             FontSize -> 12], 
            Cell[
             StyleData["Copyright", "Condensed"], 
             CellMargins -> {{8, 10}, {12, 1}}, LineSpacing -> {1, 2}, 
             FontSize -> 9], 
            Cell[
             StyleData["Copyright", "Printout"], 
             CellMargins -> {{9, 0}, {72, 4}}, FontSize -> 7]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Example"], CellMargins -> {{12, 10}, {5, 12}}, 
             LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 12}, 
             CounterIncrements -> "Example", FontFamily -> "Times", 
             FontWeight -> "Bold"], 
            Cell[
             StyleData["Example", "Presentation"], 
             CellMargins -> {{18, 10}, {8, 20}}, LineSpacing -> {1, 5}, 
             ParagraphSpacing -> {0, 12}], 
            Cell[
             StyleData["Example", "Condensed"], 
             CellMargins -> {{8, 10}, {4, 8}}, LineSpacing -> {1, 1}, 
             ParagraphSpacing -> {0, 4}], 
            Cell[
             StyleData["Example", "Printout"], 
             CellMargins -> {{9, 0}, {4, 10}}, ParagraphSpacing -> {0, 6}]}, 
           Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Definition"], CellFrame -> {{3, 0}, {0, 0}}, 
             CellMargins -> {{52, 10}, {8, 8}}, PageBreakWithin -> False, 
             GroupPageBreakWithin -> False, 
             CellLabelMargins -> {{23, Inherited}, {Inherited, Inherited}}, 
             ShowStringCharacters -> True, CounterIncrements -> "Definition", 
             FontFamily -> "Helvetica", FontWeight -> "Bold", FontColor -> 
             GrayLevel[1], Background -> RGBColor[0.2, 0.700008, 0.700008]], 
            Cell[
             StyleData["Definition", "Presentation"], 
             CellMargins -> {{62, Inherited}, {5, 12}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Definition", "Condensed"], 
             CellMargins -> {{40, 10}, {1, 4}}], 
            Cell[
             StyleData["Definition", "Printout"], 
             CellMargins -> {{44, 0}, {2, 6}}, Background -> GrayLevel[0.6]]},
            Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Theorem"], CellFrame -> {{3, 0}, {0, 0}}, 
             CellMargins -> {{52, 10}, {8, 8}}, PageBreakWithin -> False, 
             GroupPageBreakWithin -> False, 
             CellLabelMargins -> {{23, Inherited}, {Inherited, Inherited}}, 
             ShowStringCharacters -> True, CounterIncrements -> "Theorem", 
             FontFamily -> "Helvetica", FontWeight -> "Bold", FontColor -> 
             GrayLevel[1], Background -> 
             RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["Theorem", "Presentation"], 
             CellMargins -> {{62, Inherited}, {5, 12}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Theorem", "Condensed"], 
             CellMargins -> {{40, 10}, {1, 4}}], 
            Cell[
             StyleData["Theorem", "Printout"], 
             CellMargins -> {{44, 0}, {2, 6}}, Background -> GrayLevel[0.4]]},
            Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["ExerciseMain"], CellFrame -> {{6, 0}, {0, 1}}, 
             CellMargins -> {{12, Inherited}, {4, 20}}, 
             CellGroupingRules -> {"SectionGrouping", 30}, PageBreakBelow -> 
             False, CellFrameColor -> RGBColor[0.571389, 0.19675, 0.570504], 
             CounterIncrements -> "Subsection", 
             CounterAssignments -> {{"Subsubsection", 0}}, FontFamily -> 
             "Helvetica", FontSize -> 16, FontWeight -> "Bold", FontColor -> 
             RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["ExerciseMain", "Presentation"], 
             CellMargins -> {{18, 10}, {8, 32}}, LineSpacing -> {1, 2}, 
             FontSize -> 24, FontTracking -> "Condensed"], 
            Cell[
             StyleData["ExerciseMain", "Condensed"], 
             CellMargins -> {{8, Inherited}, {2, 12}}, FontSize -> 12], 
            Cell[
             StyleData["ExerciseMain", "Printout"], 
             CellMargins -> {{9, 0}, {2, 50}}, CellFrameColor -> 
             GrayLevel[0.500008], FontSize -> 14, FontTracking -> "Plain", 
             FontColor -> GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Exercise"], CellDingbat -> "\[FilledDownTriangle]", 
             CellMargins -> {{23, Inherited}, {4, 18}}, 
             CellGroupingRules -> {"SectionGrouping", 50}, PageBreakBelow -> 
             False, CounterIncrements -> "Subsubsection", FontFamily -> 
             "Times", FontSize -> 13, FontWeight -> "Bold", FontColor -> 
             RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["Exercise", "Presentation"], 
             CellMargins -> {{33, 10}, {8, 26}}, LineSpacing -> {1, 0}, 
             FontSize -> 18], 
            Cell[
             StyleData["Exercise", "Condensed"], 
             CellMargins -> {{17, Inherited}, {2, 12}}, FontSize -> 10], 
            Cell[
             StyleData["Exercise", "Printout"], 
             CellFrame -> {{0, 0}, {0.5, 0}}, CellDingbat -> None, 
             CellMargins -> {{9, 0}, {6, 20}}, FontSize -> 11, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["ExerciseText"], CellMargins -> {{62, 10}, {5, 5}}, 
             LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 8}, 
             CounterIncrements -> "ExcersiceText", FontFamily -> "Times"], 
            Cell[
             StyleData["ExerciseText", "Presentation"], 
             CellMargins -> {{74, 10}, {8, 8}}, LineSpacing -> {1, 5}, 
             ParagraphSpacing -> {0, 12}], 
            Cell[
             StyleData["ExerciseText", "Condensed"], 
             CellMargins -> {{52, 10}, {2, 2}}, LineSpacing -> {1, 1}, 
             ParagraphSpacing -> {0, 4}], 
            Cell[
             StyleData["ExerciseText", "Printout"], 
             CellMargins -> {{54, 0}, {4, 4}}, ParagraphSpacing -> {0, 6}]}, 
           Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Itemize"], CellDingbat -> "\[FilledSmallCircle]", 
             CellMargins -> {{65, Inherited}, {8, 8}}, FontFamily -> "Times", 
             FontSize -> 13, FontWeight -> "Plain", FontSlant -> "Plain", 
             FontTracking -> "Plain", 
             FontVariations -> {
              "Outline" -> False, "Shadow" -> False, "Underline" -> False}], 
            Cell[
             StyleData["Itemize", "Presentation"], 
             CellMargins -> {{Inherited, 10}, {8, 12}}, LineSpacing -> {1, 0},
              FontSize -> 18], 
            Cell[
             StyleData["Itemize", "Condensed"], 
             CellMargins -> {{Inherited, Inherited}, {2, 12}}, FontSize -> 
             10], 
            Cell[
             StyleData["Itemize", "Printout"], 
             CellMargins -> {{Inherited, 0}, {4, 20}}, FontSize -> 11]}, 
           Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["DisplayCode"], CellFrame -> 2, 
             CellMargins -> {{52, 10}, {8, 8}}, CellHorizontalScrolling -> 
             True, PageBreakWithin -> True, 
             CellLabelMargins -> {{5, Inherited}, {Inherited, Inherited}}, 
             DefaultFormatType -> DefaultInputFormatType, FormatType -> 
             InputForm, ShowStringCharacters -> True, NumberMarks -> True, 
             FontWeight -> "Bold", Background -> RGBColor[1, 1, 0]], 
            Cell[
             StyleData["DisplayCode", "Presentation"], 
             CellMargins -> {{62, Inherited}, {10, 10}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["DisplayCode", "Condensed"], 
             CellMargins -> {{40, 10}, {4, 4}}], 
            Cell[
             StyleData["DisplayCode", "Printout"], 
             CellMargins -> {{44, 0}, {6, 6}}, Background -> GrayLevel[0.8]]},
            Closed]]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["Formulas and Programming", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["DisplayFormula"], CellMargins -> {{62, 10}, {2, 10}}, 
             CellHorizontalScrolling -> True, ScriptLevel -> 0, 
             SingleLetterItalics -> True, 
             UnderoverscriptBoxOptions -> {LimitsPositioning -> True}], 
            Cell[
             StyleData["DisplayFormula", "Presentation"], 
             CellMargins -> {{74, 10}, {2, 10}}, FontSize -> 18], 
            Cell[
             StyleData["DisplayFormula", "Condensed"], 
             CellMargins -> {{52, 10}, {2, 10}}, FontSize -> 10], 
            Cell[
             StyleData["DisplayFormula", "Printout"], 
             CellMargins -> {{54, 10}, {2, 10}}, FontSize -> 10]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["ChemicalFormula"], CellMargins -> {{62, 10}, {2, 10}},
              AutoSpacing -> False, ScriptBaselineShifts -> {0.6, Automatic}, 
             ScriptLevel -> 1, SingleLetterItalics -> False, ZeroWidthTimes -> 
             True], 
            Cell[
             StyleData["ChemicalFormula", "Presentation"], 
             CellMargins -> {{74, 10}, {2, 10}}, FontSize -> 10], 
            Cell[
             StyleData["ChemicalFormula", "Condensed"], 
             CellMargins -> {{52, 10}, {2, 10}}, FontSize -> 10], 
            Cell[
             StyleData["ChemicalFormula", "Printout"], 
             CellMargins -> {{54, 10}, {2, 10}}, FontSize -> 10]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Program"], CellMargins -> {{12, 10}, {Inherited, 6}}, 
             FontFamily -> "Courier"], 
            Cell[
             StyleData["Program", "Presentation"], 
             CellMargins -> {{13, 30}, {Inherited, 4}}, FontSize -> 9.5], 
            Cell[
             StyleData["Program", "Condensed"], 
             CellMargins -> {{8, 10}, {Inherited, 4}}, FontSize -> 9.5], 
            Cell[
             StyleData["Program", "Printout"], 
             CellMargins -> {{9, 0}, {Inherited, 4}}, FontSize -> 9.5]}, 
           Closed]]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Automatic Numbering", "Section"], 
         Cell[
         "The following styles are useful for numbered equations, figures, \
etc.  They automatically give the cell a FrameLabel containing a reference to \
a particular counter, and also increment that counter.", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["NumberedEquation"], 
             CellMargins -> {{62, 10}, {Inherited, Inherited}}, 
             CellFrameLabels -> {{None, 
                Cell[
                 TextData[{"(", 
                   CounterBox["NumberedEquation"], ")"}]]}, {None, None}}, 
             DefaultFormatType -> DefaultInputFormatType, CounterIncrements -> 
             "NumberedEquation", FormatTypeAutoConvert -> False], 
            Cell[
             StyleData["NumberedEquation", "Presentation"], 
             CellMargins -> {{74, 10}, {Inherited, Inherited}}], 
            Cell[
             StyleData["NumberedEquation", "Condensed"], 
             CellMargins -> {{52, 10}, {Inherited, Inherited}}], 
            Cell[
             StyleData["NumberedEquation", "Printout"], 
             CellMargins -> {{54, 0}, {Inherited, Inherited}}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["NumberedFigure"], 
             CellMargins -> {{62, 145}, {Inherited, Inherited}}, 
             CellFrameLabels -> {{None, None}, {
                Cell[
                 TextData[{"Figure ", 
                   CounterBox["NumberedFigure"]}]], None}}, CounterIncrements -> 
             "NumberedFigure", 
             ImageMargins -> {{43, Inherited}, {Inherited, 0}}, 
             FormatTypeAutoConvert -> False], 
            Cell[
             StyleData["NumberedFigure", "Presentation"]], 
            Cell[
             StyleData["NumberedFigure", "Condensed"]], 
            Cell[
             StyleData["NumberedFigure", "Printout"]]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["NumberedTable"], 
             CellMargins -> {{62, 145}, {Inherited, Inherited}}, 
             CellFrameLabels -> {{None, None}, {
                Cell[
                 TextData[{"Table ", 
                   CounterBox["NumberedTable"]}]], None}}, TextAlignment -> 
             Center, CounterIncrements -> "NumberedTable", 
             FormatTypeAutoConvert -> False], 
            Cell[
             StyleData["NumberedTable", "Presentation"]], 
            Cell[
             StyleData["NumberedTable", "Condensed"]], 
            Cell[
             StyleData["NumberedTable", "Printout"]]}, Open]]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Headers and Footers", "Section"], 
         Cell[
          StyleData["Header"], CellMargins -> {{0, 0}, {4, 1}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 9, 
          FontSlant -> "Italic"], 
         Cell[
          StyleData["Footer"], CellMargins -> {{0, 0}, {0, 4}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 6], 
         Cell[
          StyleData["PageNumber"], CellMargins -> {{0, 0}, {4, 1}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 9, 
          FontWeight -> "Bold"]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Hyperlink Styles", "Section"], 
         Cell[
         "The cells below define styles useful for making hypertext \
ButtonBoxes.  The \"Hyperlink\" style is for links within the same Notebook, \
or between Notebooks.", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Hyperlink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, 
             FontVariations -> {"Underline" -> True}, FontColor -> 
             RGBColor[0, 0, 1], Background -> None, 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> 
               ButtonData}], 
            Cell[
             StyleData["Hyperlink", "Presentation"]], 
            Cell[
             StyleData["Hyperlink", "Condensed"]], 
            Cell[
             StyleData["Hyperlink", "Printout"], FontColor -> GrayLevel[0], 
             Background -> GrayLevel[1]]}, Open]], 
         Cell[
         "The following styles are for linking automatically to the on-line \
help system.", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["MainBookLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> GrayLevel[1], 
             Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["MainBook", #]}]& )}], 
            Cell[
             StyleData["MainBookLink", "Presentation"]], 
            Cell[
             StyleData["MainBookLink", "Condensed"]], 
            Cell[
             StyleData["MainBookLink", "Printout"], FontColor -> GrayLevel[0],
              Background -> GrayLevel[1]]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["AddOnsLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontFamily -> "Courier", 
             FontColor -> GrayLevel[1], Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["AddOns", #]}]& )}], 
            Cell[
             StyleData["AddOnsLink", "Presentation"]], 
            Cell[
             StyleData["AddOnsLink", "Condensed"]], 
            Cell[
             StyleData["AddOnLink", "Printout"], FontColor -> GrayLevel[0], 
             Background -> GrayLevel[1]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["RefGuideLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontFamily -> "Courier", 
             FontColor -> GrayLevel[1], Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["RefGuideLink", #]}]& )}], 
            Cell[
             StyleData["RefGuideLink", "Presentation"]], 
            Cell[
             StyleData["RefGuideLink", "Condensed"]], 
            Cell[
             StyleData["RefGuideLink", "Printout"], FontColor -> GrayLevel[0],
              Background -> GrayLevel[1]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["GettingStartedLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> GrayLevel[1], 
             Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["GettingStarted", #]}]& )}], 
            Cell[
             StyleData["GettingStartedLink", "Presentation"]], 
            Cell[
             StyleData["GettingStartedLink", "Condensed"]], 
            Cell[
             StyleData["GettingStartedLink", "Printout"], FontColor -> 
             GrayLevel[0], Background -> GrayLevel[1]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["OtherInformationLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> GrayLevel[1], 
             Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["OtherInformation", #]}]& )}], 
            Cell[
             StyleData["OtherInformationLink", "Presentation"]], 
            Cell[
             StyleData["OtherInformationLink", "Condensed"]], 
            Cell[
             StyleData["OtherInformationLink", "Printout"], FontColor -> 
             GrayLevel[0], Background -> GrayLevel[1]]}, Closed]]}, Closed]]},
      Open]]}, Visible -> False, FrontEndVersion -> 
  "8.0 for Microsoft Windows (32-bit) (November 7, 2010)", StyleDefinitions -> 
  "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 46, 0, 55, "Title"],
Cell[606, 22, 73, 2, 54, "Copyright"],
Cell[CellGroupData[{
Cell[704, 28, 31, 0, 54, "Section"],
Cell[738, 30, 200, 4, 25, "Text"],
Cell[941, 36, 173, 5, 31, "Itemize"],
Cell[1117, 43, 142, 4, 31, "Itemize"],
Cell[1262, 49, 341, 8, 43, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1640, 62, 33, 0, 54, "Section"],
Cell[CellGroupData[{
Cell[1698, 66, 47, 0, 44, "Subsection"],
Cell[1748, 68, 108, 3, 25, "Text"],
Cell[CellGroupData[{
Cell[1881, 75, 74, 1, 48, "Input"],
Cell[1958, 78, 493, 11, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2500, 95, 45, 0, 44, "Subsection"],
Cell[2548, 97, 155, 3, 25, "Text"],
Cell[2706, 102, 114, 3, 48, "Input"],
Cell[2823, 107, 68, 0, 25, "Text"],
Cell[2894, 109, 45, 0, 48, "Input"],
Cell[2942, 111, 37, 0, 48, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3028, 117, 42, 0, 54, "Section"],
Cell[3073, 119, 114, 2, 48, "Input"],
Cell[3190, 123, 39, 0, 25, "Text"],
Cell[3232, 125, 168, 10, 150, "Input"],
Cell[CellGroupData[{
Cell[3425, 139, 45, 0, 44, "Subsection"],
Cell[3473, 141, 99, 2, 25, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3621, 149, 48, 0, 54, "Section"],
Cell[3672, 151, 91, 2, 48, "Input"],
Cell[CellGroupData[{
Cell[3788, 157, 55, 0, 44, "Subsection"],
Cell[3846, 159, 15965, 372, 1209, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19848, 536, 129, 5, 78, "Subsection"],
Cell[19980, 543, 16631, 386, 1189, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36648, 934, 82, 1, 44, "Subsection"],
Cell[36733, 937, 2471, 92, 1561, "Input"],
Cell[39207, 1031, 98, 5, 65, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39342, 1041, 70, 0, 44, "Subsection"],
Cell[39415, 1043, 43, 0, 25, "Text"],
Cell[39461, 1045, 145, 5, 65, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39643, 1055, 48, 0, 44, "Subsection"],
Cell[39694, 1057, 242, 6, 25, "Text"],
Cell[39939, 1065, 73, 2, 48, "Input"],
Cell[CellGroupData[{
Cell[40037, 1071, 240, 8, 116, "Input"],
Cell[40280, 1081, 310, 6, 23, "Print"],
Cell[40593, 1089, 1609, 43, 225, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[42251, 1138, 46, 0, 44, "Subsection"],
Cell[CellGroupData[{
Cell[42322, 1142, 255, 8, 116, "Input"],
Cell[42580, 1152, 334, 7, 23, "Print"],
Cell[42917, 1161, 1612, 43, 225, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[44578, 1210, 46, 0, 44, "Subsection"],
Cell[CellGroupData[{
Cell[44649, 1214, 263, 9, 133, "Input"],
Cell[44915, 1225, 345, 7, 23, "Print"],
Cell[45263, 1234, 1607, 43, 225, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[46919, 1283, 73, 0, 44, "Subsection"],
Cell[CellGroupData[{
Cell[47017, 1287, 114, 4, 82, "Input"],
Cell[47134, 1293, 118, 1, 47, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[47313, 1301, 47, 0, 54, "Section"],
Cell[47363, 1303, 92, 2, 25, "Text"],
Cell[CellGroupData[{
Cell[47480, 1309, 95, 2, 48, "Input"],
Cell[47578, 1313, 152, 2, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[47779, 1321, 42, 0, 54, "Section"],
Cell[47824, 1323, 110, 3, 25, "Text"],
Cell[47937, 1328, 82, 2, 48, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48056, 1335, 43, 0, 54, "Section"],
Cell[48102, 1337, 93, 2, 25, "Text"],
Cell[CellGroupData[{
Cell[48220, 1343, 92, 2, 48, "Input"],
Cell[48315, 1347, 245, 5, 23, "Print"],
Cell[48563, 1354, 242, 4, 47, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[48854, 1364, 23, 0, 54, "Section"],
Cell[48880, 1366, 150, 4, 48, "Input"],
Cell[CellGroupData[{
Cell[49055, 1374, 138, 4, 48, "Input"],
Cell[49196, 1380, 1679, 46, 225, "Output"]
}, Open  ]],
Cell[50890, 1429, 148, 4, 48, "Input"],
Cell[CellGroupData[{
Cell[51063, 1437, 138, 4, 48, "Input"],
Cell[51204, 1443, 1679, 46, 225, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[52932, 1495, 37, 0, 54, "Section"],
Cell[CellGroupData[{
Cell[52994, 1499, 34, 0, 44, "Subsection"],
Cell[53031, 1501, 230, 9, 133, "Input"],
Cell[CellGroupData[{
Cell[53286, 1514, 171, 5, 65, "Input"],
Cell[53460, 1521, 172, 3, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53669, 1529, 181, 6, 65, "Input"],
Cell[53853, 1537, 172, 3, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54062, 1545, 172, 5, 65, "Input"],
Cell[54237, 1552, 174, 3, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54448, 1560, 182, 6, 65, "Input"],
Cell[54633, 1568, 172, 3, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54842, 1576, 96, 2, 48, "Input"],
Cell[54941, 1580, 1679, 46, 225, "Output"]
}, Open  ]],
Cell[56635, 1629, 42, 0, 48, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[56726, 1635, 41, 0, 54, "Section"],
Cell[CellGroupData[{
Cell[56792, 1639, 1667, 39, 348, "Input"],
Cell[58462, 1680, 2897, 75, 223, "Output"]
}, Open  ]],
Cell[61374, 1758, 127, 3, 25, "Text"],
Cell[61504, 1763, 2244, 49, 146, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[63785, 1817, 27, 0, 54, "Section"],
Cell[63815, 1819, 406, 10, 89, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
